/**
 * Main search orchestrator that manages different search strategies
 */
export class SearchManager {
    config;
    strategy;
    constructor(config, strategy) {
        this.config = config;
        this.strategy = strategy;
    }
    async search(query, entities, options, project) {
        // Handle exact search mode
        if (options?.searchMode === 'exact') {
            return this.exactSearch(query, entities);
        }
        // Handle fuzzy search mode
        const threshold = options?.fuzzyThreshold || this.config.threshold;
        try {
            // Try database search first if available
            if (this.strategy.canUseDatabase()) {
                return await this.strategy.searchDatabase(query, threshold, project);
            }
        }
        catch (error) {
            console.warn('Database search failed, falling back to client-side:', error);
            // Fall back to client-side if database search fails
            if (this.config.clientSideFallback) {
                return this.strategy.searchClientSide(entities, query);
            }
            throw error;
        }
        // Use client-side search
        return this.strategy.searchClientSide(entities, query);
    }
    exactSearch(query, entities) {
        const lowerQuery = query.toLowerCase();
        return entities.filter(e => {
            const entityTags = e.tags || [];
            return (e.name.toLowerCase().includes(lowerQuery) ||
                e.entityType.toLowerCase().includes(lowerQuery) ||
                e.observations.some(o => o.toLowerCase().includes(lowerQuery)) ||
                entityTags.some(tag => tag.toLowerCase().includes(lowerQuery)));
        });
    }
}
//# sourceMappingURL=search-manager.js.map